import moment from 'moment'
import store from '@/store'
/**
 * 通用js方法封装处理
 * Copyright
 */

const baseURL = process.env.VUE_APP_BASE_API

// 日期格式化
export function parseTime(time, pattern) {
  if (arguments.length === 0 || !time) {
    return null
  }
  const format = pattern || '{y}-{m}-{d} {h}:{i}:{s}'
  let date
  if (typeof time === 'object') {
    date = time
  } else {
    if (typeof time === 'string' && /^[0-9]+$/.test(time)) {
      time = parseInt(time)
    } else if (typeof time === 'string') {
      time = time.replace(new RegExp(/-/gm), '/')
    }
    if (typeof time === 'number' && time.toString().length === 10) {
      time = time * 1000
    }
    date = new Date(time)
  }
  const formatObj = {
    y: date.getFullYear(),
    m: date.getMonth() + 1,
    d: date.getDate(),
    h: date.getHours(),
    i: date.getMinutes(),
    s: date.getSeconds(),
    a: date.getDay()
  }
  const time_str = format.replace(/{(y|m|d|h|i|s|a)+}/g, (result, key) => {
    let value = formatObj[key]
    // Note: getDay() returns 0 on Sunday
    if (key === 'a') {
      return ['日', '一', '二', '三', '四', '五', '六'][value]
    }
    if (result.length > 0 && value < 10) {
      value = '0' + value
    }
    return value || 0
  })
  return time_str
}

// 表单重置
export function resetForm(refName) {
  if (this.$refs[refName]) {
    this.$refs[refName].resetFields()
  }
}

// 添加日期范围
export function addDateRange(params, dateRange) {
  var search = params
  search.beginTime = ''
  search.endTime = ''
  if (dateRange != null && dateRange !== '') {
    search.beginTime = this.dateRange[0]
    search.endTime = this.dateRange[1]
  }
  return search
}

// 按照月份添加
export function addDateRangeMothon(params, dateRange) {
  var search = params
  if (dateRange !== null && dateRange !== '') {
    this.dateRange = this.dateRange.split('/')
    search.operTime = this.dateRange[0] + '' + this.dateRange[1]
  }
  return search
}
export function addDateLoginMothon(params, dateRange) {
  var search = params
  if (dateRange !== null && dateRange !== '') {
    this.dateRange = this.dateRange.split('/')
    search.loginTime = this.dateRange[0] + '' + this.dateRange[1]
  }
  return search
}

// 回显数据字典
export function selectDictLabel(datas, value) {
  var actions = []
  Object.keys(datas).map(key => {
    if (datas[key].dictValue === '' + value) {
      actions.push(datas[key].dictLabel)
      return false
    }
  })
  if (actions.length === 0) {
    return '-'
  }
  return actions.join('')
}

// 通用下载方法
export function download(fileName) {
  window.location.href =
    baseURL +
    '/authority/common/download?fileName=' +
    encodeURI(fileName) +
    '&delete=' +
    true
}

// 字符串格式化(%s )
export function sprintf(str) {
  var args = arguments
  var flag = true
  var i = 1
  str = str.replace(/%s/g, function() {
    var arg = args[i++]
    if (typeof arg === 'undefined') {
      flag = false
      return ''
    }
    return arg
  })
  return flag ? str : ''
}

// 转换字符串，undefined,null等转化为""
export function praseStrEmpty(str) {
  if (!str || str === 'undefined' || str === 'null') {
    return ''
  }
  return str
}

/**
 * 构造树型结构数据
 * @param {*} data 数据源
 * @param {*} id id字段 默认 'id'
 * @param {*} parentId 父节点字段 默认 'parentId'
 * @param {*} children 孩子节点字段 默认 'children'
 * @param {*} rootId 根Id 默认 0
 */
export function handleTree(data, id, parentId, children, rootId) {
  id = id || 'id'
  parentId = parentId || 'parentId'
  children = children || 'children'
  rootId = rootId || 0
  // 对源数据深度克隆
  const cloneData = JSON.parse(JSON.stringify(data))
  // 循环所有项
  const treeData = cloneData.filter(father => {
    const branchArr = cloneData.filter(child => {
      // 返回每一项的子级数组
      return (father[id] + '') === (child[parentId] + '')
    })
    branchArr.length > 0 ? (father.children = branchArr) : ''
    // 返回第一层
    return father[parentId] === rootId
  })
  return treeData !== '' ? treeData : data
}

/**
 * @author WangXinYu
 * @describe 格式化日期到时分
 * @param { string } dateString 日期
 * @return
 **/
export function formatDateStandard(dateString) {
  if (!dateString || dateString === '') {
    return ''
  }
  return moment(dateString).format('YYYY-MM-DD HH:mm')
}
// 判断有无权限方法
export function includePermission(accessName) {
  if (!accessName) { return false }
  if (!store.getters.permissions) { return false }
  return (store.getters.permissions.includes(accessName) || store.getters.permissions.includes('*:*:*'))
}
// 随机生成32位字符串
export const randomString = () => {
  const len = 32
  const $chars =
    'ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz2345678'
  const maxPos = $chars.length
  let pwd = ''
  for (let i = 0; i < len; i++) {
    pwd += $chars.charAt(Math.floor(Math.random() * maxPos))
  }
  return pwd
}
// 校验出错 跳转到第一个出错的地方
import Vue from 'vue'
export const scrollToError = () => {
  Vue.prototype.$nextTick(() => {
    // 锚点 scrollIntoView 方法
    const target = document.createElement('a')
    target.style.position = 'absolute'
    target.style.top = '-6.4vh' // 减去头部定位的60px高度
    const targetClass = 'el-form-item__error' // 需要滚动到容器class
    const firstErrorItem = document.querySelector(`.${targetClass}`)
    if (!firstErrorItem) {
      return
    }
    // 插入 目标锚点
    firstErrorItem.parentNode.appendChild(target)
    // 查询input
    const inputFirst = firstErrorItem.parentNode.querySelector('input')
    if (inputFirst) {
      // 设置监听事件
      const scrollStart = function() {
        target.scrollIntoView()
      }
      // input只读并且 可编辑 触发点击事件
      if (inputFirst.readOnly && !inputFirst.disabled) {
        // 加入监听事件
        inputFirst.addEventListener('click', scrollStart, {
          once: true
        })
        inputFirst.click()
        // 点击后马上移除 防止 再次点击时 滚动
        // inputFirst.removeEventListener('click', scrollStart)
        // 不只读并且可输入 触发聚焦事件
      } else if (!inputFirst.readOnly && !inputFirst.disabled) {
        // 加入监听事件
        inputFirst.addEventListener('focus', scrollStart, {
          once: true
        })
        inputFirst.focus()
        // 点击后马上移除 防止 再次点击时 滚动
        // inputFirst.removeEventListener('focus', scrollStart)
      } else {
        target.scrollIntoView({
          behavior: 'smooth'
        })
      }
    } else {
      target.scrollIntoView({
        behavior: 'smooth'
      })
    }
    Vue.prototype.$sleep(300).then(() => { firstErrorItem.parentNode.removeChild(target) })
  })
}
