import { randomString } from '@/utils/common'

const errCode = e => {
  const errMessage = {
    '511': '您的签名未生效，请联系管理员'
  }
  if (errMessage[e.errcode]) {
    e.errmsg = errMessage[e.errcode]
    return e
  }
  return e
}

/**
 * @author: Joey Wong
 * @description: 盖章操作
 * @param: {Object} options 操作设置
 * @param: {Function} okCallback 成功回调
 * @param: {Object} errCallback 失败回调
 * @return:
 **/
const SERVER_URL = process.env['VUE_APP_SIGNATURE']
export const creatSignature = (
  options,
  okCallback,
  cancelCallback,
  errorCallback
) => {
  const { keySn, signName, position, documentId, protectedItems } = options
  const documentIdArray = documentId.split(',')
  const positionArray = position.split(',')
  console.log(positionArray)
  // eslint-disable-next-line no-undef
  Signature.init({
    // 初始化属性
    // keysn: '46D307326484FCAC',
    // keysn: '12010119900180909',
    certType: 'server', // 设置证书在签章服务器
    sealType: 'server', // 设置印章从签章服务器取
    serverUrl: SERVER_URL, //
    // documentid: documentId, // 设置文档ID
    // documentname: documentName, // 设置文档名称
    LANGUAGE: 'zh_CN', // 语言设置
    pw_timeout: 's1800', // s：秒；h:小时；d:天
    showProtectedBtn: false, // 设置是否显示保护项按钮
    moveable: false,
    signname: signName, // 签章名称
    scaleImage: 0.4
    // showSealsDlg: 'auto' // 是否选择dialog
  })
  // eslint-disable-next-line no-undef
  const signatureCreator = Signature.create({
    keysn: keySn // 可以通过密钥盘序号或者用户编码获取签章。
    // authcode: 'test'
    // usercode: userCode // 获取签章服务器用户编码为“test”的印章
  })
  signatureCreator.run({
    // protectedItems: protectedItems, // 设置定位页面DOM的id，自动查找ID，自动获取保护DOM的kg-desc属性作为保护项描述，value属性为保护数据。不设置，表示不保护数据，签章永远有效。
    autoCert: false,
    // position: positionArray[0],
    offsetX: 100,
    offsetY: 0,
    protectedItems: protectedItems,
    // iframeID: 'templateIframe',
    batchSignature: [
      {
        documentid: documentIdArray[0],
        documentName: documentIdArray[0] + '-table',
        protectedItems: protectedItems, // 设置定位页面DOM的id，自动查找ID，自动获取保护DOM的kg-desc属性作为保护项描述，value属性为保护数据。不设置，表示不保护数据，签章永远有效。
        position: positionArray[0]
      },
      {
        documentid: documentIdArray[1],
        documentName: documentIdArray[1] + '-process',
        protectedItems: protectedItems, // 设置定位页面DOM的id，自动查找ID，自动获取保护DOM的kg-desc属性作为保护项描述，value属性为保护数据。不设置，表示不保护数据，签章永远有效。
        position: positionArray[1]
      }
    ],
    okCall: function(fn, image) {
      // 点击确定后的回调方法，this为签章对象 ,签章数据撤销时，将回调此方法，需要实现签章数据持久化（保存数据到后台数据库）,保存成功后必须回调fn(true/false)渲染签章到页面上
      console.log('盖章ID：' + this.getSignatureid())
      console.log('盖章数据：' + this.getSignatureData())
      const resData = {
        signatureId: this.getSignatureid(),
        signatureData: this.getSignatureData()
      }
      if (okCallback) {
        okCallback(resData)
      }
      fn(true)
    },
    cancelCall: function() {
      // 点击取消后的回调方法
      console.log('取消！')
      if (cancelCallback) {
        cancelCallback()
      }
    },
    errorCall: function(e) {
      console.log(e)
      if (errorCallback) {
        errorCallback(errCode(e))
      }
    }
  })
}

/**
 * @author: Joey Wong
 * @description: 保存盖章操作
 * @param: {Object} options 操作设置
 * @param: {Function} okCallback 成功回调
 * @param: {Object} errCallback 失败回调
 * @return:
 **/
export const saveSign = (options, okCallBack) => {
  const { keySn, signName, documentId, documentName } = options
  // eslint-disable-next-line no-undef
  Signature.init({
    // 初始化属性
    // keysn: '46D307326484FCAC',
    keysn: keySn,
    certType: 'server', // 设置证书在签章服务器
    sealType: 'server', // 设置印章从签章服务器取
    serverUrl: SERVER_URL, //
    // documentid: documentId, // 设置文档ID
    // documentname: documentName, // 设置文档名称
    LANGUAGE: 'zh_CN', // 语言设置
    pw_timeout: 's1800', // s：秒；h:小时；d:天
    showProtectedBtn: true, // 设置是否显示保护项按钮
    moveable: false,
    signname: signName, // 签章名称
    showSealsDlg: true // 是否选择dialog
  })
  // eslint-disable-next-line no-undef
  const signatureCreator = Signature.create()
  // eslint-disable-next-line no-undef
  const list = Signature.list
  for (const key in list) {
    // eslint-disable-next-line no-unused-vars
    const tt = list[key]
    signatureCreator.saveSignature(
      tt.signatureid,
      key,
      list[key].getSignatureData(),
      function(ret) {
        if (okCallBack) {
          okCallBack()
        }
      }
    )
  }
}

/**
 * @author: Joey Wong
 * @description: 回显签章
 * @param: {Object} options 操作设置
 * @param: {Function} okCallback 成功回调
 * @param: {Object} errCallback 失败回调
 * @return:
 **/
export const initSignature = (options, okCallBack) => {
  const { signData, scaleImage } = options
  // eslint-disable-next-line no-undef
  Signature.init({
    certType: 'server', // 设置证书在签章服务器
    sealType: 'server', // 设置印章从签章服务器取
    serverUrl: SERVER_URL, //
    LANGUAGE: 'zh_CN', // 语言设置
    pw_timeout: 's1800', // s：秒；h:小时；d:天
    showProtectedBtn: true, // 设置是否显示保护项按钮
    moveable: false,
    icon_remove: false,
    icon_move: false,
    icon_move_donot: false,
    icon_sign: false // 数字签名按钮隐藏显示，缺省显示
  })
  // eslint-disable-next-line no-unused-vars
  // eslint-disable-next-line no-undef
  // eslint-disable-next-line no-undef
  Signature.loadSignatures(signData, null, function() {
    if (okCallBack) {
      okCallBack()
    }
  })
}

/**
 * @author: Joey Wong
 * @description: 显示签章
 * @param: {}
 * @return:
 **/
export const showSignature = () => {
  // eslint-disable-next-line no-undef
  Signature.show()
}
/**
 * @author: Joey Wong
 * @description: 通过signatureid，显示、隐藏指定的电子签章。
 * @param: {}
 * @return:
 **/
export const showAndHideSignature = (signatureId, visible) => {
  // eslint-disable-next-line no-undef
  Signature.showAndHideBySignatureId(signatureId, visible)
}
/**
 * @author: Joey Wong
 * @description: 批量验证签章
 * @param: {}
 * @return:
 **/
export const verifySignatureByList = strarr => {
  // eslint-disable-next-line no-undef
  const list = Signature.list
  const list2 = []
  // const strarr = ['155627555338199417', '155650753606110295']
  for (const key in list) {
    const signature = list[key]
    if (strarr.indexOf(key) > -1) {
      list2.push(signature)
    }
  }
  //
  // eslint-disable-next-line no-undef
  const signatureData = Signature.verifySignByList(list2)
  for (let i = 0; i < signatureData.length; i++) {
    const signature = signatureData[i]
    console.log(signature) // 获取修改的保护项
    console.log(signature.modifiedItems) // 获取修改的保护项
  }
}

/**
 * @author: Joey Wong
 * @description: 删除签章的方法
 * @param: {Array} documentIdArray 文档Id的数组
 * @param: {Array} signatureIdArray 签章Id的数组
 * @return:
 **/
export const removeSignature = ({ documentIdArray, signatureIdArray } = {}) => {
  // eslint-disable-next-line no-undef
  const signatureCreator = Signature.create()
  signatureCreator.removeSignature(documentIdArray[0], signatureIdArray[0])
  signatureCreator.removeSignature(documentIdArray[1], signatureIdArray[1])
}

/**
 * @author: Joey Wong
 * @description: 批量审批中处理数据
 * @param: {Array} resData 需要处理的数组
 * @return:
 **/
export const batchApprovalDataFormat = ({ resData = [] } = {}) => {
  return new Promise((resolve, reject) => {
    try {
      // * 需要返回的数据 参考createSignature中的 batchSignature 字段
      let batchSignatureArray = []
      // * 生成对应的dom
      resData.forEach(item => {
        const { resultFieldAndPermissionDTOList } = item
        const processKey = resultFieldAndPermissionDTOList[0].processKey
        const instanceId = resultFieldAndPermissionDTOList[0].instanceId
        const userTaskId = resultFieldAndPermissionDTOList[0].taskId
        // * 生成盖章的documentId，由processKey和instanceId组成
        const tempId = `${processKey}-${instanceId}-${userTaskId}`
        // * 表格签章的文档id
        const tableDocumentId = `${tempId}-table-${randomString()}`
        // * 审批流程签章的文档id
        const processDocumentId = `${tempId}-process-${randomString()}`
        // * 表格签章位置id
        const tablePositionId = `approval-record-pos-table-${randomString()}`
        // * 审批流程签章位置id
        const processPositionId = `approval-record-pos-process-${randomString()}`
        // * 保护的数据项
        const protectedItems = []

        // * 处理 resultFieldAndPermissionDTOList 成为保护数据

        for (const resultFieldAndPermissionDTOListElement of resultFieldAndPermissionDTOList) {
          const {
            isRead,
            businessId,
            fieldName,
            fieldValue
          } = resultFieldAndPermissionDTOListElement
          if (isRead && Number(isRead) === 1) {
            protectedItems.push({
              field: businessId,
              desc: fieldName,
              value: fieldValue ? fieldValue + '' : ''
            })
          }
        }

        const tempItemArray = [
          {
            documentid: tableDocumentId,
            documentName: tableDocumentId + '-table',
            protectedItems: protectedItems, // 设置定位页面DOM的id，自动查找ID，自动获取保护DOM的kg-desc属性作为保护项描述，value属性为保护数据。不设置，表示不保护数据，签章永远有效。
            position: tablePositionId
          },
          {
            documentid: processDocumentId,
            documentName: processDocumentId + '-process',
            protectedItems: protectedItems, // 设置定位页面DOM的id，自动查找ID，自动获取保护DOM的kg-desc属性作为保护项描述，value属性为保护数据。不设置，表示不保护数据，签章永远有效。
            position: processPositionId
          }
        ]
        batchSignatureArray = [...batchSignatureArray, ...tempItemArray]
      })
      resolve(batchSignatureArray)
    } catch {
      reject()
    }
  })
}

/**
 * @author: Joey Wong
 * @description: 批量审批签章操作
 * @param: {}
 * @return:
 **/
export const batchCreateSignature = ({
  protectedDataArray = [],
  signName = '',
  keySn = '',
  okCallback,
  cancelCallback,
  errorCallback
} = {}) => {
  // eslint-disable-next-line no-undef
  Signature.init({
    // 初始化属性
    // keysn: '46D307326484FCAC',
    // keysn: '12010119900180909',
    certType: 'server', // 设置证书在签章服务器
    sealType: 'server', // 设置印章从签章服务器取
    serverUrl: SERVER_URL, //
    // documentid: documentId, // 设置文档ID
    // documentname: documentName, // 设置文档名称
    LANGUAGE: 'zh_CN', // 语言设置
    pw_timeout: 's1800', // s：秒；h:小时；d:天
    signname: signName, // 签章名称
    scaleImage: 0.4,
    showProtectedBtn: false, // 设置是否显示保护项按钮
    moveable: false,
    icon_remove: false,
    icon_move: false,
    icon_move_donot: false,
    icon_sign: false // 数字签名按钮隐藏显示，缺省显示
    // showSealsDlg: 'auto' // 是否选择dialog
  })
  // eslint-disable-next-line no-undef
  const signatureCreator = Signature.create({
    keysn: keySn // 可以通过密钥盘序号或者用户编码获取签章。
    // authcode: 'test'
    // usercode: userCode // 获取签章服务器用户编码为“test”的印章
  })
  signatureCreator.run({
    // protectedItems: protectedItems, // 设置定位页面DOM的id，自动查找ID，自动获取保护DOM的kg-desc属性作为保护项描述，value属性为保护数据。不设置，表示不保护数据，签章永远有效。
    autoCert: false,
    // position: positionArray[0],
    offsetX: 100,
    offsetY: 0,
    // protectedItems: protectedItems,
    // iframeID: 'templateIframe',
    batchSignature: protectedDataArray,
    okCall: function(fn, image) {
      // 点击确定后的回调方法，this为签章对象 ,签章数据撤销时，将回调此方法，需要实现签章数据持久化（保存数据到后台数据库）,保存成功后必须回调fn(true/false)渲染签章到页面上
      console.log(this)
      console.log('盖章ID：' + this.getSignatureid())
      console.log('盖章数据：' + this.getSignatureData())
      const documentId = this.oriSignatureData.documentid
      const documentArray = documentId.split('-')
      const processKey = documentArray[0]
      const instanceId = documentArray[1]
      const userTaskId = documentArray[2]
      const docmId = Object.keys(this.oriSignatureData.position)[0]
      const resData = {
        processKey: processKey,
        instanceId: instanceId,
        taskId: userTaskId,
        documentId: documentId,
        signature: this.getSignatureData(),
        signatureId: this.getSignatureid(),
        docmId: docmId
      }
      if (okCallback) {
        okCallback(resData)
      }
      fn(true)
    },
    cancelCall: function() {
      // 点击取消后的回调方法
      console.log('取消！')
      if (cancelCallback) {
        cancelCallback()
      }
    },
    errorCall: function(e) {
      console.log(e)
      if (errorCallback) {
        errorCallback(errCode(e))
      }
    }
  })
  console.log(protectedDataArray)
}
