import axios from 'axios'
import { MessageBox, Message } from 'element-ui'
import store from '@/store'
import Qs from 'qs'
import { getToken, setToken, refreshToken, removeToken } from '@/utils/auth'
import SM4 from './gmUtil'
import errorCode from '@/utils/errorCode'

axios.defaults.headers['Content-Type'] = 'application/json;charset=utf-8'
// 创建axios实例
const service = axios.create({
  // axios中请求配置有baseURL选项，表示请求URL公共部分
  baseURL: process.env.VUE_APP_BASE_API,
  // 超时
  timeout: 100000,
  headers: { 'clientType': 'fy_suit' }
})
// let loadingInstance = ''
// request拦截器
service.interceptors.request.use(config => {
  // 请求开始 loading 开始
  /* const loadingCount = store.state.app.loadingCount
  console.log(loadingCount)
  if (loadingCount === 0) {
    loadingInstance = Loading.service({
      lock: true
    })
  }*/
  // store.dispatch('app/setLoadingCount', loadingCount + 1)
  // 是否需要设置 token
  const isToken = (config.headers || {}).isToken === false
  if (getToken() && !isToken) {
    config.headers['Authorization'] = 'Bearer ' + getToken() // 让每个请求携带自定义token 请根据实际情况自行修改
  }
  // 参数发出前统一进行加密 白名单内的不进行加密
  console.log(config)

  // if(config.data) {
  //   config.data = Qs.stringify(config.data, { arrayFormat: 'repeat' })
  // }
  return config
}, error => {
  console.log(error)
  Promise.reject(error)
})
// 响应拦截器
service.interceptors.response.use(res => {
  /* // 判断 loading 状态
  const loadingCount = store.state.app.loadingCount
  const oneLessCount = loadingCount - 1
  if (oneLessCount === 0) {
    setTimeout(() => {
      loadingInstance.close()
    }, 300)
  }*/
  // store.dispatch('app/setLoadingCount', oneLessCount)
  // 未设置状态码则默认成功状态
  const code = res.data.code || 200
  // 获取错误信息
  const message = errorCode[code] || res.data.msg || res.data.message || errorCode['default']
  // status为401 code为e004则重新获取token
  if ({ 401: true, 'E004': true }[code] === true) {
    Message({
      type: 'error',
      message: '登陆状态已过期'
    })
    setTimeout(() => {
      removeToken()
      window.location.href = `${process.env.VUE_APP_LOGIN}?redirect=&reLocation=${process.env.VUE_APP_SUITW}`// 跳轉到登錄
    }, 3000)
  } else if (code === 500 || code === 400) {
    const errMsg = res.data.data || res.data.message || res.data.msg
    Message({
      message: errMsg,
      type: 'error'
    })
    return Promise.reject(new Error(message))
  } else if (code !== 200 && code !== 'E004') {
    Message({
      message: message,
      type: 'error',
      duration: 5 * 1000
    })
    return Promise.reject('error')
  } else if (code !== 'E004') {
    if (res.config.url.indexOf('downLoadSignaturesSample') > -1 || res.config.url.indexOf('downLoadSignatures') > -1) {
      return res
    } else {
      return res.data
    }
  }
},
error => {
  console.log('err' + error)
  let message = error.message
  if (error.message === 'Network Error') {
    message = '当前网络未连接，请检查您的网络设置'
  }
  Message({
    message: message,
    type: 'error',
    duration: 5 * 1000
  })
  return Promise.reject(error)
}
)
export default service
